@@ -6,7 +6,15 @@ from w3bsite.classes.views.defaults import *
 import importlib
 class Documentations(View):
 	def __init__(self):
-		a=1
+		
+		# docs.
+		DOCS = {
+			"module":"website.views.docs", 
+			"initialized":True,
+			"description":[], 
+			"chapter": "Docs", }
+
+		#
 	def chapter(self,
 		# the chapter title.
 		title="Chapter Title",
@@ -50,7 +58,7 @@ class Documentations(View):
 			"parameters_count":len(parameters),
 			"code_areas":self.__assign_element_ids__(code_areas, key="element_id"),
 			"code_areas_count":len(code_areas),
-			"element_id":self.__generate_element_id__(),
+			"element_id":self.__generate_element_id__("sect_"),
 			"type":type,
 		}
 	def subsection(self,
@@ -926,6 +934,477 @@ class Documentations(View):
 		return variable
 		#
 
+	# build docs from code.
+	def build(self,
+		# the package name.
+		package=None,
+		# the root path (optional).
+		root=None,
+		# the package's description (leave "" or None to ignore).
+		description="",
+		# the package's install command (leave "" or None to ignore).
+		installation=None,
+		# the package's installation description (leave "" or None to ignore).
+		installation_description="",
+		# include system functions (aka: def __somefunc__).
+		system_functions=False,
+		# include different path's or readme str.
+		include=[],
+		# banned sub paths.
+		banned=[".docs/build.py"],
+		# banned names.
+		banned_names=["__main__.py", "utils.py"],
+		# banned basenames.
+		banned_basenames=["utils", "__pycache__", ".legacy"],
+		# banned class types.
+		banned_class_types=["Exception"],
+		# the banned classes.
+		banned_classes=[],
+		# the banned functions.
+		banned_functions=[],
+		# the export path.
+		readme=None,
+		# the examples export path.
+		examples=None, 
+		# the chapters export path.
+		chapters=None,
+		# the replacemenets.
+		replacements={},
+	):	
+		# funcs.
+		def clean_params(params):
+			clean = params.replace('(self)', '()').replace('self  ,', '').replace('self ,', '').replace('self,', '').replace("\n)", " )")
+			while True:
+				if clean[:len("( ")] == "( ":
+					clean = "("+clean[2:]
+				else: break
+			return clean
+		def build_description(info):
+			description = info["description"]
+			if description in [[], ""]:
+				l = info['module']
+				if l == None:
+					l = info['name']
+				if info["type"] == "class":
+					description = f"The <attribute>{l}</attribute> object class."
+				else:
+					description = f"The <attribute>{l}</attribute> function."
+			for from_, to_ in {
+				"(required)":"",
+				"(required)":"",
+			}.items():
+				description = description.replace(from_, to_)
+			return description
+		def build_params(info):
+			parameters = []
+			cleaned = clean_params(info["parameters"])
+			param_name, param_number, param_format, param_value = None, None, None, None
+			comments = ""
+			required = False
+			if cleaned != "()":
+				dict_open = 0
+				for line in cleaned[1:-1].split("\n"):
+					for _ in range(100):
+						if len(line) > 0 and line[0] == " ": line = line[1:]
+						else: break
+					new_open, new_closed = False, False
+					for i in line:
+						if i == "{": 
+							if dict_open == 0: new_open = True
+							dict_open += 1
+						elif i == "}": 
+							dict_open -= 1
+							if dict_open == 0: new_closed = True
+					if line not in ["", "}", "]", ")"] and (new_open or dict_open == 0) and not new_closed:
+						if len(line) > 0 and line[0] == "#":
+							comment = True
+						else:
+							comment = False
+						if comment: 
+							comments += line+"\n"
+						else:
+							# handle line / str.
+							def handle(string, param_format=None, comments=""):
+								if "=" in string:
+									param_name = string.split("=")[0]
+									param_value = string.split("=")[1].split(",")[0]
+									required = "(required)" in comments or "(required)" in comments
+								elif ":" in string:
+									param_name = string.split(":")[0]
+									required = True
+									param_format = string.split(":")[1].split(",")[0]
+								else:
+									param_name = string.split(",")[0]
+									required = True
+									param_format = None
+								return param_name, required, param_format
+							# slice param format from comments.
+							def parse_parameter_format_from_comments(param_format, comments=""):
+								formats = ["None", "NoneType", "bool", "Boolean", "str", "String", "int", "Integer", "float", "dict", "Dictionary", "list", "Array", "ResponseObject", "OutputObject", "FilePath", "File", "Directory"]
+								depth, last_sliced = 1, None
+								for _ in range(100):
+									sliced = String(comments).slice_tuple(depth=depth)[1:-1].replace(" ","").replace("	","").split(",")
+									if sliced == last_sliced: break
+									if len(sliced) > 0 and sliced[0] == "#":
+										try: 
+											param_number = int(sliced[1:])
+										except:
+											param_number = None
+									for l_format in formats:
+										if l_format in sliced:
+											param_format = sliced
+											break
+									if param_format != None: break
+									last_sliced = sliced
+									depth += 1
+								return param_format
+							# slice param format from default value.
+							def slice_param_format_from_default_value(param_format, param_value):
+								if param_value in ["True", "False"]:
+									param_format = ["bool"]
+								elif len(param_value) > 0 and param_value[0] in ["'", '"']:
+									param_format = ["str"]
+								elif len(param_value) > 0 and param_value[0] in ["["]:
+									param_format = ["list"]
+								elif len(param_value) > 0 and param_value[0] in ["{"]:
+									param_format = ["dict"]
+								elif len(param_value) > 0 and param_value[0] in ["("]:
+									param_format = ["slice"]
+								elif param_value in ["dev0s.defaults.options.log_level"]:
+									param_format = ["int"]
+								else:
+									if "." in str(param_value):
+										try:
+											float(param_value)
+											param_format = ["float"]
+										except: a=1
+									else:
+										try:
+											int(param_value)
+											param_format = ["int"]
+										except: a=1
+								return param_format, param_value
+							# handle line.
+							param_name, required, param_format = handle(line, param_format=param_format, comments=comments)
+						if param_name != None:
+
+							# slice param format from comments.
+							if param_format == None:
+								param_format = parse_parameter_format_from_comments(param_format, comments=comments)
+
+							# slice param format from default value.
+							if param_format == None and param_value not in [None,"None"]:
+								param_format, param_value = slice_param_format_from_default_value(param_format, param_value)
+
+							# unkown format.
+							if param_format == None:
+								param_format = "unknown"
+								if "--hide-alerts" not in sys.argv:
+									dev0s.response.log(f"Missing parameter format declaration from {info['type']} [{info['name']}] ({info['path']}).", mode="alert")
+							if isinstance(param_format, (str,String)):
+								param_format = [param_format]
+
+							# create description from comments.
+							param_description = []
+							for line in comments.split("\n"):
+								for _ in range(100):
+									if len(line) > 0 and line[0] in ["#", " "]: line = line[1:]
+									else: break
+								if line not in [""]:
+									param_description.append(str(String(line).capitalized_word()))
+
+							# append.
+							parameters.append(self.parameter(
+								id=param_name,
+								format=param_format,
+								required=required,
+								folded=not required,
+								description=param_description,))
+
+							# reset.
+							param_name, param_number, param_format, param_value = None, None, None, None
+							comments = ""
+							required = False
+			# exact copy to add remainders.
+			if param_name != None:
+				if param_format == None:
+					formats = ["None", "NoneType", "bool", "Boolean", "str", "String", "int", "Integer", "float", "dict", "Dictionary", "list", "Array", "ResponseObject", "OutputObject", "FilePath", "File", "Directory"]
+					depth, last_sliced = 1, None
+					for _ in range(100):
+						sliced = String(comments).slice_tuple(depth=depth)[1:-1].replace(" ","").replace("	","").split(",")
+						if sliced == last_sliced: break
+						if len(sliced) > 0 and sliced[0] == "#":
+							try: 
+								param_number = int(sliced[1:])
+							except:
+								param_number = None
+						for l_format in formats:
+							if l_format in sliced:
+								param_format = sliced
+								break
+						if param_format != None: break
+						last_sliced = sliced
+						depth += 1
+				if param_format == None and param_value not in [None,"None"]:
+					if param_value in ["True", "False"]:
+						param_format = ["bool"]
+					elif len(param_value) > 0 and param_value[0] in ["'", '"']:
+						param_format = ["str"]
+					elif len(param_value) > 0 and param_value[0] in ["["]:
+						param_format = ["list"]
+					elif len(param_value) > 0 and param_value[0] in ["{"]:
+						param_format = ["dict"]
+					elif len(param_value) > 0 and param_value[0] in ["("]:
+						param_format = ["slice"]
+					elif param_value in ["dev0s.defaults.options.log_level"]:
+						param_format = ["int"]
+					else:
+						if "." in str(param_value):
+							try:
+								float(param_value)
+								param_format = ["float"]
+							except: a=1
+						else:
+							try:
+								int(param_value)
+								param_format = ["int"]
+							except: a=1
+				if param_format == None:
+					param_format = "unknown"
+				if isinstance(param_format, (str,String)):
+					param_format = [param_format]
+				param_description = []
+				for line in comments.split("\n"):
+					for _ in range(100):
+						if len(line) > 0 and line[0] in ["#", " "]: line = line[1:]
+						else: break
+					if line not in [""]:
+						param_description.append(str(String(line).capitalized_word()))
+				parameters.append(self.parameter(
+					id=param_name,
+					format=param_format,
+					required=required,
+					folded=not required,
+					description=param_description,))
+				param_name, param_number, param_format, param_value = None, None, None, None
+				comments = ""
+				required = False
+			# end copy.
+			return parameters
+		def build_title(info, class_=None):
+			if info["type"] in ["class"]:
+				return info["raw_name"].split("-")[0]
+			else:
+				if class_ != None:
+					return class_+"."+info["raw_name"].split("-")[0]
+				else:
+					return info["raw_name"].split("-")[0]
+		def build_code(info):
+			if info["type"] == "class":
+				if "." in info["raw_name"]: l = info["raw_name"].split(".")[len(info["raw_name"].split("."))-1]
+				else: l = info["raw_name"]
+				initialized_name = String(l).variable_format()
+				l = info["module"]
+				if l == None: l = info["name"]
+				if info["initialized"]:
+					code = initialized_name+" = " + l
+				else:
+					# check if not a class without __init__
+					if "self" not in info["parameters"]:
+						code = initialized_name+" = " + l
+					else:
+						code = initialized_name+" = " + l + clean_params(str(info["parameters"]))
+			elif info["type"] == "property":
+				if "." in info["raw_name"]: l = info["raw_name"].split(".")[len(info["raw_name"].split("."))-1]
+				else: l = info["raw_name"]
+				initialized_name = String(l).variable_format()
+				code = initialized_name+" = " + info["name"]
+			else:
+				code = info["return"]+" = " + info["name"] + clean_params(str(info["parameters"]))
+			return code
+
+		# vars.
+		chapters_export = chapters
+		chapters = []
+		if installation_description in ["", None]: installation_description = ""
+		if isinstance(installation_description, (str,String)): installation_description = [installation_description]
+		# build code examples.
+		docs, _ = Python(path=root).build_readme(
+			# the package name.
+			package=package,
+			# the root path (optional).
+			root=root,
+			# the package's description (leave "" or None to ignore).
+			description=description,
+			# the package's install command (leave "" or None to ignore).
+			installation=installation,
+			# include system functions (aka: def __somefunc__).
+			system_functions=system_functions,
+			# include different path's or readme str.
+			include=include,
+			# banned sub paths.
+			banned=banned,
+			# banned names.
+			banned_names=banned_names,
+			# banned basenames.
+			banned_basenames=banned_basenames,
+			# banned class types.
+			banned_class_types=banned_class_types,
+			# the banned classes.
+			banned_classes=banned_classes,
+			# the banned functions.
+			banned_functions=banned_functions,
+			# the export path.
+			readme=readme,
+			# the examples export path.
+			examples=examples, 
+			# the replacements.
+			replacements=replacements,)
+
+		# loader.
+		loader = Loader("Creating website chapters")
+		
+		# build installation chapter.
+		sections = []
+		if "inc-package-manager --install" in installation:
+			sections += [
+				# installation.
+				self.section(
+					title="Installation",
+					description=[
+						f"Easily install <italic>{package}</italic> with <italic>inc-package-manager</italic>.",
+					],
+					attributes=[],
+					parameters=[],
+					code_areas=[
+						self.code_area(
+							title="install",
+							content={
+								"cli":"""inc-package-manager --install vserver""",
+								"bash":"""curl -X GET https://api.vandenberghinc.com/packages/download/\?package=vserver&api_key=***API_KEY*** -J -o ~/Downloads/vserver.zip """,
+						}),
+						
+					]),
+			]
+		else:
+			if installation_description == [""]:
+				installation_description = [f"Install the {package} library."]
+			sections += [
+				# installation.
+				self.section(
+					title="Installation",
+					description=installation_description,
+					attributes=[],
+					parameters=[],
+					code_areas=[
+						self.code_area(
+							title="install",
+							content={
+								"bash":installation,
+						}),
+						
+					]),
+			]
+
+		# append installation chapter.
+		chapters.append(self.chapter(
+			# the chapter title.
+			title="Installation",
+			# the chapter's sections.
+			sections=sections,
+			# whether the chapter is folded in the leftbar or not.
+			folded=True,))
+
+		# build code chapters.
+		for chapter, items in docs.items():
+			sections = []
+			for id, info in items.items():
+
+				# first indent functions.
+				if info["type"].lower() == "function":
+
+					# section.
+					sections.append(self.section(
+						title=build_title(info),
+						description=build_description(info),
+						attributes=[],
+						parameters=build_params(info),
+						code_areas=[
+							self.code_area(
+								title="func",
+								content={
+									"python":build_code(info),
+								}
+							),
+						],
+					))
+
+				# object classes.
+				elif info["type"].lower() == "class":
+
+					# iterate properties.
+					attributes = []
+					for func_info in info["functions"]:
+						if func_info["type"] in ["property"]:
+							attributes.append(self.attribute(
+								id=func_info["name"],
+								format=func_info["return"],
+								folded=True,
+								description=func_info["description"],))
+
+					# section.
+					title = build_title(info)
+					sections.append(self.section(
+						title=title,
+						description=build_description(info),
+						attributes=attributes,
+						parameters=build_params(info),
+						code_areas=[
+							self.code_area(
+								title="init",
+								content={
+									"python":build_code(info),
+								}
+							),
+						],
+					))
+
+					# iterate functions.
+					for func_info in info["functions"]:
+						if func_info["type"] not in ["property"] and "__init__" not in func_info["name"]:
+
+							# func section.
+							sections.append(self.subsection(
+								title=build_title(func_info, class_=title),
+								description=build_description(func_info),
+								attributes=[],
+								parameters=build_params(func_info),
+								code_areas=[
+									self.code_area(
+										title="func",
+										content={
+											"python":build_code(func_info),
+										}
+									),
+								],
+							))
+
+			# append chapter.
+			chapters.append(self.chapter(
+				# the chapter title.
+				title=chapter,
+				# the chapter's sections.
+				sections=sections,
+				# whether the chapter is folded in the leftbar or not.
+				folded=True,))
+
+		# save.
+		if chapters_export != None:
+			loader.mark(new_message=f"Saving created website docs [{chapters_export}].")
+			Files.save(chapters_export, "chapters = "+str(chapters))
+		loader.stop()
+
+
 	#
 
 # the documentation view.
@@ -948,7 +1427,17 @@ class DocumentationView(View):
 		colors={},
 		# the w3bsite.Website object.
 		website=None,
+		# overwrite maintenance (bool) (leave None to use website.maintenance).
+		maintenance=None,
 	):
+
+		# docs.
+		DOCS = {
+			"module":"website.views.DocumentationView", 
+			"initialized":False,
+			"description":[], 
+			"chapter": "Views", }
+
 		# defaults.
 		View.__init__(self,
 			base=base,
@@ -960,6 +1449,8 @@ class DocumentationView(View):
 			# the view type.
 			type="DocumentationView",)
 		self.website = website
+		self.maintenance = maintenance
+		if self.maintenance == None: self.maintenance = self.website.maintenance
 
 		# check library.
 		if not Files.exists("__defaults__/static/media"): os.mkdir("__defaults__/static/media")
@@ -1027,7 +1518,7 @@ class DocumentationView(View):
 		if response.success:
 			response = self.website.users.get_api_key(email=response.email)
 			if response.success: api_key = response.api_key
-		if self.website.maintenance: return self.maintenance(request)
+		if self.maintenance: return self.maintenance(request)
 		return self.render(request=request, html=f"w3bsite/classes/views/html/documentation_view.html", template_data={
 			"API_KEY":api_key,
 			"URL":self.url,
